package com.ncs.occ.test;

import org.apache.camel.EndpointInject;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.Produce;
import org.apache.camel.ProducerTemplate;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.builder.xml.XPathBuilder;
import org.apache.camel.component.mock.MockEndpoint;
import org.apache.camel.test.spring.CamelSpringTestSupport;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;


public class CxfConsumerSchemaValidationTest extends CamelSpringTestSupport {
    @Produce(uri = "direct:start")
    protected ProducerTemplate template;

    @EndpointInject(uri = "mock:result")
    protected MockEndpoint resultEndpoint;

    protected static final Logger LOGGER = LoggerFactory.getLogger(CxfConsumerSchemaValidationTest.class);
    protected final String simpleEndpointAddress = "http://localhost:29909/calculatorService";

    private String sumResponse = "<cal:sumResponse xmlns:cal=\"http://www.ncs.com/occ/calculator\"><total>#total#</total></cal:sumResponse>";

    private String sumRequestUnWrapped = "<cal:sumRequest xmlns:cal=\"http://www.ncs.com/occ/calculator\"><x>1111</x><y>1</y></cal:sumRequest>";

    private String sumRequestWrapped = "<cal:sum xmlns:cal=\"http://www.ncs.com/occ/calculator\"><x>1111</x>"
        + "<y>1</y></cal:sum>";

    @Override
    protected AbstractApplicationContext createApplicationContext() {
        return new ClassPathXmlApplicationContext("META-INF/spring/cxf-test-context.xml");
    }

    @Override
    protected RouteBuilder createRouteBuilder() {
        return new RouteBuilder() {

            @Override
            public void configure() throws Exception {

                from(
                    "cxf://http://localhost:30011/calculatorService?wsdlURL=wsdl/calculator_services_unwrapped.wsdl&dataFormat=PAYLOAD")
                        .streamCaching().to("log:info").process(new SumProcessor());

                from(
                    "cxf://http://localhost:30012/calculatorService?wsdlURL=wsdl/calculator_services_wrapped.wsdl&dataFormat=PAYLOAD")
                        .streamCaching().to("log:info").process(new SumProcessor());

            }
        };
    }

    @Test
    public void testUnWrappedMessage() throws Exception {
        template.setDefaultEndpointUri("jetty:http://localhost:30000/test");
        template.sendBody(sumRequestUnWrapped);
    }

    @Test
    public void testWrappedMessage() throws Exception {
        
//        Thread.sleep(1000000);

        template.setDefaultEndpointUri("jetty:http://localhost:30000/test");
        template.sendBody(sumRequestWrapped);
    }

    class SumProcessor implements Processor {

        @Override
        public void process(Exchange exchange) throws Exception {
            Object request = exchange.getIn().getBody(String.class);

            LOGGER.info("calculator service request: {}", request);

            // attempt 1) applying XPath to exchange.getIn().getBody()
            String x = XPathBuilder.xpath("//*[name()='x']/text()").evaluate(context, request);
            String y = XPathBuilder.xpath("//*[name()='y']/text()").evaluate(context, request);

            String response = sumResponse.replace("#total#", String.valueOf(Integer.parseInt(x) + Integer.parseInt(y)));
            exchange.getIn().setBody(response);

            LOGGER.info("calculator service response: {}", response);
        }

    }

}
